/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.entity.grammardefnode;

import java.util.LinkedList;
import java.util.List;

import net.sf.parser4j.generator.service.GrammarDefNodeVisitException;
import net.sf.parser4j.generator.service.grammardefnode.IGrammarDefVisitor;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class RulesDef implements IGrammarDefNode {

	private List<RuleDef> list = new LinkedList<RuleDef>();// NOPMD

	private String whiteSpaceNonTerminalName;

	public RulesDef(final RuleDef ruleDef) {
		list.add(ruleDef);
	}

	public RulesDef(final RulesDef rulesDef, final RuleDef ruleDef) {
		list.addAll(rulesDef.list);
		list.add(ruleDef);
	}

	public void setWhiteSpaceNonTerminalName(
			final String whiteSpaceNonTerminalName) {
		this.whiteSpaceNonTerminalName = whiteSpaceNonTerminalName;
	}

	public String getWhiteSpaceNonTerminalName() {
		return whiteSpaceNonTerminalName;
	}

	public void addAll(final RulesDef toAdd) {
		list.addAll(toAdd.list);
	}

	public List<RuleDef> getList() {
		return list;
	}

	public void accept(final IGrammarDefVisitor visitor)
			throws GrammarDefNodeVisitException {
		visitor.beginVisit(this);
		for (RuleDef ruleDef : list) {
			ruleDef.accept(visitor);
		}
		visitor.endVisit(this);
	}

	@Override
	public String toString() {
		return "RulesDef";
	}
}
