/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.entity.grammardefnode;

import java.util.Map;
import java.util.TreeMap;

import net.sf.parser4j.generator.service.GrammarDefNodeVisitException;
import net.sf.parser4j.generator.service.grammardefnode.IGrammarDefVisitor;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class RulesGroupListDef implements IGrammarDefNode {

	private Map<String, RulesDef> map = new TreeMap<String, RulesDef>(); // NOPMD

	public RulesGroupListDef(final RulesGroupListDef rulesGroupListDef,
			final RulesGroupDef rulesGroupDef) {
		map.putAll(rulesGroupListDef.map);
		final String whiteSpaceNonTerminalName = rulesGroupDef
				.getWhiteSpaceNonTerminalName();
		final RulesDef existing = map.get(whiteSpaceNonTerminalName);
		final RulesDef toAdd = rulesGroupDef.getRulesDef();
		toAdd.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		if (existing == null) {
			map.put(whiteSpaceNonTerminalName, toAdd);
		} else {
			existing.addAll(toAdd);
		}
	}

	public RulesGroupListDef(final RulesGroupDef rulesGroupDef) {
		final RulesDef toAdd = rulesGroupDef.getRulesDef();
		final String whiteSpaceNonTerminalName = rulesGroupDef
				.getWhiteSpaceNonTerminalName();
		toAdd.setWhiteSpaceNonTerminalName(whiteSpaceNonTerminalName);
		map.put(whiteSpaceNonTerminalName, toAdd);
	}

	public Map<String, RulesDef> getMap() {
		return map;
	}

	public void accept(final IGrammarDefVisitor visitor)
			throws GrammarDefNodeVisitException {
		visitor.beginVisit(this);
		for (RulesDef rulesDef : map.values()) {
			rulesDef.accept(visitor);
		}
		visitor.endVisit(this);
	}

	@Override
	public String toString() {
		return "RulesGroupListDef";
	}
}
