/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.parser.entity;

/**
 * grammar node type enumeration
 * 
 * @author luc peuvrier
 * 
 */
public enum EnumNodeType {

	/**
	 * node for concat of node
	 */
	CONCAT(false),
	/**
	 * node for one of alternative son node
	 */
	ALTERNATIVE(false),
	/**
	 * empty node
	 */
	EMPTY(true),
	/**
	 * character terminal node
	 */
	CHARACTER_TERMINAL(true),
	/**
	 * string terminal node
	 */
	STRING_TERMINAL(true),
	/**
	 * null grammar node
	 */
	NULL(true/* dummy value */);

	private final boolean terminal;

	private EnumNodeType(final boolean terminal) {
		this.terminal = terminal;
	}

	public boolean isTerminal() {
		if (equals(NULL)) {
			throw new UnsupportedOperationException("is null");
		}
		return terminal;
	}

	public boolean isNonTerminal() {
		if (equals(NULL)) {
			throw new UnsupportedOperationException("is null");
		}
		return true ^ terminal;
	}
}
