/*
 * Copyright 2008 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generateandparsetest.test;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.Writer;

import net.sf.parser4j.parser.entity.ParseResult;
import net.sf.parser4j.parser.entity.parsenode.IParseNode;
import net.sf.parser4j.parser.service.HasAmbiguityParserException;
import net.sf.parser4j.parser.service.Parser;
import net.sf.parser4j.parser.service.ParserException;
import net.sf.parser4j.parser.service.parsenode.ParseTreeToXML;

/**
 * 
 * @author luc peuvrier
 * 
 */
public class AmbiguousGrammarTest extends AbstractGrammarTest {

	public void testAmbiguousGrammar() throws ParserException, IOException,
			HasAmbiguityParserException {// NOPMD

		final Parser parser = createParser("ambiguous.xml");
		/*
		 * parsing of "a+a*a"
		 */
		final StringReader reader = new StringReader("a+a*a");

		final ParseResult result = parser.parse(reader, true, false);
		final IParseNode parseNode = result.getResultParseNode();
		assertFalse("must have one result\n" + result, result.isInError());
		assertTrue("must have alternative for ambiguity", parser
				.hasAlternativeForAmbiguity());

		final File outputFile = new File("runtime/parsetree.xml");
		ParseTreeToXML.getInstance().toXml(outputFile, parseNode,
				nonTerminalMap.getNonTerminalByIdentifierMap());
		final String toString = parsingToStringUtil
				.parseTreeToString(parseNode);
		final FileOutputStream fileOutputStream =
		/**/new FileOutputStream("runtime/logs/parse_node.txt");
		final Writer writer = new OutputStreamWriter(fileOutputStream, "UTF8");
		writer.write(toString);
		writer.close();

		final String expected =
		/**/"begin ALTERNATIVE Start(65542)\n" +
		/**/" begin ALTERNATIVE Expression(65541)\n" +
		/**/"  begin alternative #0 Expression\n" +
		/**/"   begin CONCAT ExpressionMul(65540)\n" +
		/**/"    begin ALTERNATIVE Expression(65541)\n" + // NOPMD
				/**/"     begin CONCAT ExpressionAdd(65538)\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" + // NOPMD
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"      begin CHARACTER_TERMINAL add(65537) '+'\n" +
				/**/"      end add\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" +
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"     end ExpressionAdd\n" +
				/**/"    end Expression\n" +
				/**/"    begin CHARACTER_TERMINAL mul(65539) '*'\n" +
				/**/"    end mul\n" +
				/**/"    begin ALTERNATIVE Expression(65541)\n" +
				/**/"     begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"     end ntA\n" +
				/**/"    end Expression\n" +
				/**/"   end ExpressionMul\n" +
				/**/"  end alternative #0\n" +
				/**/"  begin alternative #1 Expression\n" +
				/**/"   begin CONCAT ExpressionAdd(65538)\n" +
				/**/"    begin ALTERNATIVE Expression(65541)\n" +
				/**/"    begin CHARACTER_TERMINAL add(65537) '+'\n" +
				/**/"    begin ALTERNATIVE Expression(65541)\n" +
				/**/"     begin CONCAT ExpressionMul(65540)\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" +
				/**/"      begin CHARACTER_TERMINAL mul(65539) '*'\n" +
				/**/"      end mul\n" +
				/**/"      begin ALTERNATIVE Expression(65541)\n" +
				/**/"       begin CHARACTER_TERMINAL ntA(65536) 'a'\n" +
				/**/"       end ntA\n" +
				/**/"      end Expression\n" +
				/**/"     end ExpressionMul\n" +
				/**/"    end Expression\n" +
				/**/"   end ExpressionAdd\n" +
				/**/"  end alternative #1\n" +
				/**/" end Expression\n" +
				/**/"end Start\n";

		assertEquals("not same output", expected, parsingToStringUtil
				.parseTreeToString(parseNode));
	}

	public void newLine(final int line) {
		//
	}
}
