/*
 * Copyright 2009 Luc Peuvrier
 * 
 * This file is part of parser4j.
 *
 * parser4j is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * parser4j is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with parser4j.  If not, see <http://www.gnu.org/licenses/>.
 */
package net.sf.parser4j.generator.service;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;
import net.sf.parser4j.generator.entity.ToGeneratorFile;
import net.sf.parser4j.kernelgenerator.service.GeneratorException;
import net.sf.parser4j.parser.entity.ParseResult;

import org.apache.log4j.Logger;

/**
 * test generator
 * 
 * @author luc peuvrier
 * 
 */
public class GeneratorTest extends TestCase {

	private static final Logger _log = Logger.getLogger(GeneratorTest.class);

	private static final String MUST_SUCCEED = "must succeed";

	private static final ToGeneratorFile toGeneratorFile =
	/**/ToGeneratorFile.getInstance();

	private static final ParserDataGenerator parserDataGenerator =
	/**/ParserDataGenerator.getInstance();

	public void testG1() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g1.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g1_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g2.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG3() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g3.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g3_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG4() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g4.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g4_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG5() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g5.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g5_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG1w() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g1w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g1w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2w() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g2w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG2w1() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g2w_1.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g2w_1_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG3w() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g3w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g3w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG4w() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g4w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g4w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	public void testG5w() throws GeneratorException, IOException {
		final String inputGrammarFileName = "g5w.txt";
		final ParseResult parseResult = generate(inputGrammarFileName,
				"g5w_out.txt");
		assertFalse(MUST_SUCCEED, parseResult.isInError());
	}

	private ParseResult generate(final String inputGrammarFileName,
			final String outFileName) throws GeneratorException, IOException {
		final File inputFile = toGeneratorFile.createFile(inputGrammarFileName);
		final File outFile = new File("runtime/" + outFileName);
		final ParseResult parseResult = parserDataGenerator.generate(inputFile,
				null/* htmlOutputFile */, null/* parserDataOutputFile */,
				""/* matchMgrPackageName */, null/* grammarDefTreeTextFile */,
				outFile/* grammarTreeTextFile */);
		if (parseResult.isInError()) {
			_log.error(parseResult.toString());
		} else {
			_log.info(parseResult.toString());
			final String out = readFile(outFile);
			final File outExpecteFile = toGeneratorFile.createFile(outFileName);
			final String outExpected = readFile(outExpecteFile);
			assertEquals("must be same", outExpected, out);
		}
		return parseResult;
	}

	private String readFile(final File file) throws IOException {
		final InputStream inputStream = new FileInputStream(file);
		final StringBuilder stringBuilder = new StringBuilder();
		int readed;
		while ((readed = inputStream.read()) != -1) {// NOPMD
			stringBuilder.append((char) readed);
		}
		inputStream.close();
		return stringBuilder.toString();
	}

}
